/* GSI - general sound interface  [-*- C++ -*-]
 *
 * gsi_commands.h
 *
 * Copyright (C) 1996,1997,1998 W.H.Scholten
 *
 * ** Sound server command specification 0.33 **
 * $Log: gsi_commands.h,v $
 * Revision 1.9  1998/05/28 09:37:22  wouter
 * small updates
 *
 * Revision 1.8  1998/05/24 13:15:41  wouter
 *
 */

/* You can use, redistribute and/or modify this software under the terms of
 * the CGPL version 0.4 or later (see the file LICENSE.CGPL for details)
 */



#ifndef _gsi_commands_h
#  define _gsi_commands_h

/*---------------------------------------------------------------------------*
 *
 *	Server commands
 *
 *---------------------------------------------------------------------------*/
//Notation: after a command are denoted the arguments and how many bytes are
//used for the command and each argument

/*__________________CONTROL COMMANDS___________________*/
/* Commands 192-255 */


//Meta commands
#define SND_OPEN_SERVER             255 /* 1 (+ program name?)for multiple use, or just return an id (byte) which the server needs for each command to know who sent the command */
#define SND_CLOSE_SERVER            254
#define SND_PAUSE_ALL               253 /* 1 pause all sound for the given application */
#define SND_CONTINUE_ALL            252 /* 1 */
//PAUSE_DSP/PAUSE_SYNTH/PAUSE_CD

//Server control
// Rename: SND_AUTO_SET_DSP ?
// Better: SND_AUTO_CONFIG_DSP
#define SND_USE_BEST_DSP_SETTINGS   248 /* 1 e.g. when there are no sounds playing, and a new sound is started, dsp settings are set to those of the sound to be played */
#define SND_SET_DSP_SPEED           247 /* <speed> 1 2 */
#define SND_SET_DSP_CHANNELS        246 /* <channels> 1 1 */
#define SND_SET_DSP_BYTES           245 /* <type> 1 1 */
#define SND_SET_DSP_FRAGSIZE        244 /* <size> 1 1 */

#define SND_GRAB_DSP                243 /* 1 also switches 'autodsp' off */
/* grabs the DSP and sets the speed, channels etc. as set by earlier commands. Sending this when the DSP is already grabbed will set the dsp to the new settings (if any were set). */

#define SND_RELEASE_DSP             242 /* 1 also switches 'autodsp' off */

#define SND_AUTO_DSP                241 /* 1 automatically grabs the DSP when it receives a play command, releases it when all sounds have been played; particularly usefull for apps that only want to emit beeps and such and don't want to hog the DSP device */

// Unimplemented:
#define SND_GRAB_SYNTH              240 /* 1 */
#define SND_RELEASE_SYNTH           239 /* 1 */
#define SND_AUTO_SYNTH              238 /* is this useful? need to restore all patches etc. */


#define SND_SET_BASE_WIDTH          237 /* <width =0-256> 1 2 */
/* width = 0 will effectively give mono sound when the device is stereo */
/* width = 256 will give maximal stereo effect */

#define SND_SET_MAX_AUTO_CHANNELS   231 /* <n> 1 1  .. This sets the maximum simultaneous number of playing sounds on the stack */
// What about SND_SET_MAX_STATIC_CHANNELS, so set the max numner of simultaneous playing sounds on channels (ignore play commands when the limit is reached)

#define SND_CALLBACK_ON             230 /* switches callback on for all commands until a SND_CALLBACK_OFF */
#define SND_CALLBACK_OFF            229 /* switches callback off again */


// Sound dir, sound file, load sounds, free sounds etc.
#define SND_LOAD_SOUND_LIST         223
#define SND_FREE_SOUND_LIST         222 /* 1 must also free sounds that have been loaded */
#define SND_LOAD_LISTED             221 /* 1 load all sounds from the sound list */
#define SND_FREE_LISTED             220 /* 1 free paths for listed sounds */

#define SND_LOAD_SOUND              219 /* <n> 1 1 (load slot 'n', from the list) */
#define SND_LOAD_SOUND_FILE         218 /* <slot> <string> 1 1 strlen+1, string gives file path (string ends with a 0) */
#define SND_LOAD_SOUND_DATA         217 /* <slot> <speed> <type> <channels> <data_length> <data> 1 1 4 1 1 4 data_length */

#define SND_SET_SLOT_INFO           228  /* <slot> <speed> <type> <channels> (<LSB/MSB> ??) 1 1 2 1 1 */ //Also set the size ? For copy past stuff?
#define SND_GET_SLOT_INFO           227  /* <slot> 1 1 => SND_GET_SLOT_INFO <slot> <speed> <type> <channels> <size> (<LSB/MSB>??)  1 1 2 1 1 4 returns the info to the program via a callback */

#define SND_FREE_SOUND              216 /* <slot> 1 1 */

#define SND_SET_SOUND_DIR           215 /* <string> 1 strlen+1, For files in list */

#define SND_AUTO_FREE_SOUND         214 /* <slot> 1 1  Free memory used for a sound when it is not playing on any stack/channel (load again when needed) */


//These 5 are not yet implemented:
#define SND_SET_SOUND_TRANSFORM     207 /* ?? */
#define SND_SET_SOUND               206 /* set waveform etc, instead of SND_LOAD_SOUND; + data (wave type(sine, sawtooth, square, ..) , freq, ... ) */
//rename? SND_SET_SOUND_TYPE / SND_SET_SOUND_WAVE
#define SND_SET_STREAM_SIZE         205 /* <size> 1 4, sounds larger than the given size are not loaded completely into memory, but loaded 8K (other value?) at a time, good value is 64K; rename? */
//Other useful things?
#define SND_LOCK_SOUNDS             204 /* Don't allow new sounds to be loaded */
#define SND_UNLOCK_SOUNDS           203 /* Allow new sounds to be loaded */

#define SND_SYNC                    200 /* get all data, then hard_flush the status */

//Unimplemented:
//Patch loading (will make a program device dependant)
#define SND_LOAD_PATCH_DATA         199 /* <bank> <len> <data>  1 1 4 len */
#define SND_LOAD_PATCH_FILE         198 /* <bank> <string>      1 1 strlen+1 */
//Midi loading
#define SND_LOAD_SONG_DATA          197 /* <len> <data>  1 4 len */
#define SND_LOAD_SONG_FILE          196 /* <string>      1 strlen+1 */

#define SND_SET_SOUND_SPEED         195 /* <speed> 1 2  i.e. the representation of mach 1 */


/* Sound recording, preliminary spec */
#define SND_RECORD_SLOT             192 /* <slot> <cmd> <...>  1 1 1 ... */
/* to define settings, set the DSP to what you want, and SND_GRAB_DSP, then record */
/* Sub commands cmd: */
#  define SND_RECORD_NOW         0 /* 1 */
#  define SND_RECORD_AT          1 /* <sec> <musec> 1 4 4 (i.e. gettimeofday values) */
#  define SND_RECORD_DURATION    2 /* <musec> 1 4 */
#  define SND_RECORD_AUTO_START  3 /* 1 , i.e. start when there's really something, otherwise throw it away */
#  define SND_RECORD_AUTO_STOP   4 /* once started, stop when there's silence */
#  define SND_RECORD_AUTO_THRESHOLD 5 /* anything below this threshold is considered random noise, hence silence */

#define SND_DUPLEX_SLOT             193 /* <slot> */
/* This disables recording? Or do we limit .. */



/* Data sharing/copying between clients */
#define GSI_GET_CLIENT_ID           201 /* 1 returns GSI_CALLBACK_CLIENT_ID <id> 1 4 */
#define GSI_COPY_SLOT               202 /* <client id> <slot 1> <slot 2> 1 1 4 1 copies a slots data (or its characteristics) from the other client's slot (slot 1) to slot 2 */



/*__________________PLAY SOUND COMMANDS___________________*/
/* Commands 0-39 */


//Stack playing sounds:
#define SND_PLAY                      0 /* <slot> 1 1 */
//#define SND_PLAY_IF_NOT_PLAYING     0
#define SND_OVERWRITE                 1 /* <slot> 1 1 */
#define SND_REPEAT                    2 /* <slot> 1 1 repeat a sound until killed */
#define SND_OVERWRITE_REPEAT          3 /* <slot> 1 1 */
//currently not implemented:
#define SND_LOOP                      4 /* <slot> <loopno> 1 1 1, loopno=0 gives repeat forever? */
#define SND_OVERWRITE_LOOP            5 /* <slot> <loopno> 1 1 1, ditto */
#define SND_PLAY_WAVE                 7 /* <wave_type> <freq> <vol> 1 1 2 1 */
//rename SND_PLAY_NOTE ?
//#define SND_NOTE                   10 /* Use wave table: chan, freq, dur */
//#define SND_FREQ                   11 /* for note */
//#define SND_HOLD                    8 /* 'or': to start the sound paused */
//synchronize with other sounds etc? SND_SYNC <num_chan's> <ch 1> <ch 2> ...


//#define SND_KILL_THESE             12 /* <slot> 1 1 (kill all sounds of the given type) */
#define SND_KILL                     13 /* <slot> 1 1 (kill last started 'slot') */
#define SND_PAUSE                    14 /* <slot> 1 1 (pause last started 'slot') */
#define SND_CONTINUE                 15 /* <slot> 1 1 (ditto) */


#define SND_CHANNEL                  16 /* 'OR' this with above stack commands to get a channel command (mainly meant for INTERNAL use) */
//Channel playing sounds:
#define SND_CHANNEL_PLAY             16 /* <ch> <slot> 1 1 1 play a sound on a channel */
#define SND_CHANNEL_REPEAT           17 /* <ch> <slot> 1 1 1 */
#define SND_CHANNEL_OVERWRITE        18 /* <ch> <slot> 1 1 1 */
#define SND_CHANNEL_OVERWRITE_REPEAT 19 /* <ch> <slot> 1 1 1 */
//#define SND_CHANNEL_KILL_THESE     28 /* <slot> 1 1 (kill all channels playing the give slot) */
#define SND_CHANNEL_KILL             29 /* <ch> 1 1 */
#define SND_CHANNEL_PAUSE            30 /* <ch> 1 1 */
#define SND_CHANNEL_CONTINUE         31 /* <ch> 1 1 */


#define SND_KILLALL                  32 /* 1 */

/*__________________MIXER COMMANDS___________________*/
#define SND_SET_MIXER                33 /* <mixer channel> <mixer vol> 1 1 1 mixer volumes are 0-255 */
//arguments of mixer channel are GSI_MASTER_VOLUME etc (see below)
#define SND_GET_MIXER                34 /* <mixer channel> 1 1 => <SND_STATUS_MIXER> <mixer_ch> <mixer vol>  1 1 1  */
//Of is deze niet nodig? (stuur waarden na init vd client of verandering door een client automatisch)

/*__________________SONG PLAYING COMMANDS___________________*/
#define SND_PLAY_SONG                36
#define SND_STOP_SONG                37
#define SND_CONTINUE_SONG            38
#define SND_MIDI_CONTROL             39 /* <action | chn> <parm> 1 1 1 */
//This controls midi parameters for a channel (0-15)
//action is a MIDI controller: 
//#define SND_MIDI_SET_BANK_SELECT      0x00
//#define SND_MIDI_SET_MODWHEEL         0x10
//#define SND_MIDI_SET_BREATH           0x20
///* undefined 0x30 */
//#define SND_MIDI_SET_FOOT             0x40
//#define SND_MIDI_SET_PORTAMENTO_TIME  0x50
//#define SND_MIDI_SET_DATA_ENTRY       0x60
//#define SND_MIDI_SET_MAIN_VOLUME      0x70
//#define SND_MIDI_SET_BALANCE          0x80
///* undefined 0x90 */
//#define SND_MIDI_SET_PAN              0xa0
//#define SND_MIDI_SET_EXPRESSION       0xb0
///* undefined 0xc0 */
///* undefined 0xd0 */
///* undefined 0xe0 */
///* undefined 0xf0 */


/*__________________SOUND MODIFICATION COMMANDS___________________*/
// Commands 40-191

// The observer (40-63)


//Observer movement: NOTE: these do NOT need CURRENT!
#define SND_SET_O_X                  45 // <x> 1 2
#define SND_SET_O_Z                  46 // <z> 1 2
#define SND_SET_O_Y                  47 // <y> 1 2

#define SND_SET_O_ANGLE              48 // <fi> 1 1
#define SND_SET_O_FI                 48 // <fi> 1 1  alias for ANGLE
#define SND_SET_O_THETA              49 // <th> 1 1
#define SND_SET_O_R                  50 // <r> 1 2
//#define SND_SET_O_DANGLE          122 // <dfi> 1 1
//angle of a sound is not needed for objects (if a sound has a direction, e.g. a sound cone, then handle that in the main program). The angle above for a sound actually places the object relative to the observer
#define SND_ADJUST_O_X               51 // <x> 1 2
#define SND_ADJUST_O_Z               52 // <y> 1 2
#define SND_ADJUST_O_Y               53 // <z> 1 2
#define SND_ADJUST_O_FI              54 // <fi> 1 1
#define SND_ADJUST_O_THETA           55 // <th> 1 1
#define SND_ADJUST_O_R               56 // <r> 1 2

#define SND_SET_O_DX                 57 // <dx> 1 2
#define SND_SET_O_DZ                 58 // <dz> 1 2
#define SND_SET_O_DY                 59 // <dy> 1 2

#define SND_O_ECHO_ON                60 // <dt> <dvolume> 1 1 1
#define SND_O_ECHO_OFF               61 // 1
#define SND_O_DOPPLER_ON             62 // 1
#define SND_O_DOPPLER_OFF            63 // 1



// Sounds modification commands: 64-191


#define SND_CURRENT                  32 // 'OR' with a command to access the last started sound

// Commands:  64- 95, ( 96-127=CURRENT version) Position stuff
// Commands: 128-159, (160-191=CURRENT version) All else, speeds, volume ...

// Between brackets: arguments when using SND_CURRENT (i.e. no <slot>):
// Note that the ADJUST_X etc position settings will use the old position for doppler calculations
// The SET_X etc do not (to initialise the postion somewhere)
#define SND_SET_X                    64 // <slot> <x>  1 1 2  { <x>  1 2 }
#define SND_SET_Z                    65 // <slot> <z>  1 1 2  { <z>  1 2 }
#define SND_SET_Y                    66 // <slot> <y>  1 1 2  { <z>  1 2 }
// SND_SET_(CHANNEL_)ANGLE also sets new radius; use old radius if r=0?
#define SND_SET_ANGLE                67 // <slot> <angle> 1 1 1 { <angle> 1 1 }
#define SND_SET_ANGLE_R              68 // <slot> <angle> <r> 1 1 1 2 { <angle> <r> 1 1 2 }
#define SND_SET_FI                   67 // <slot> <angle> 1 1 1 
#define SND_SET_THETA                69 // <slot> <theta> 1 1 1 
#define SND_SET_R                    70 // <slot> <r>     1 1 2 
//This one sets the angle and the distance to the default distance.
//#define SND_SET_ANGLE_DEFAULT_R      77 // <slot> <angle> 1 1 1 { <angle> 1 1 }
#define SND_SET_PAN                  71 // <slot> <pan position> 1 1 2  -256 gives a sound fully left, 0 centered, 256 fully right. This disables 3d effects for the given sound (doppler, motion)


#define SND_ADJUST_X                 72
#define SND_ADJUST_Z                 73
#define SND_ADJUST_Y                 74
#define SND_ADJUST_ANGLE             75
#define SND_ADJUST_ANGLE_R           76
#define SND_ADJUST_FI                75 // alias for ANGLE
#define SND_ADJUST_THETA             77
#define SND_ADJUST_R                 78


// 'OR' SND_SET_* with SND_CHANNEL to set transforms for the channels
//  SND_SET_* + SND_CHANNEL = SND_SET_CHANNEL_*
//Direct use of the following is also possible:
#define SND_SET_CHANNEL_X            80 // <ch> <x> 1 1 2     { <x> 1 2  }
#define SND_SET_CHANNEL_Z            81 // <ch> <z> 1 1 2     { <z> 1 2  }
#define SND_SET_CHANNEL_Y            82 // <ch> <y> 1 1 2     { <z> 1 2  }
#define SND_SET_CHANNEL_ANGLE        83 // <ch> <angle> 1 1 1 { <angle> 1 1 }
//or new:
#define SND_SET_CHANNEL_ANGLE_R      84 // <ch> <angle> <r> 1 1 1 2 { <angle> 1 1 }
#define SND_SET_CHANNEL_FI           83 // alias for ANGLE
#define SND_SET_CHANNEL_THETA        85
#define SND_SET_CHANNEL_R            86
// This one sets the angle and the distance to the default distance.
//#define SND_SET_CHANNEL_ANGLE_DEFAULT_R  88 // <slot> <angle> 1 1 1 { <angle> 1 1 }
#define SND_SET_CHANNEL_PAN          87 // <ch> <pan position> 1 1 2  -256 gives a sound fully left, 0 centered, 256 fully right. This disables 3d effects on the given channel (doppler, motion)


#define SND_ADJUST_CHANNEL_X         88
#define SND_ADJUST_CHANNEL_Z         89
#define SND_ADJUST_CHANNEL_Y         90
#define SND_ADJUST_CHANNEL_ANGLE     91
#define SND_ADJUST_CHANNEL_ANGLE_R   92
#define SND_ADJUST_CHANNEL_FI        91 // alias for ANGLE
#define SND_ADJUST_CHANNEL_THETA     93
#define SND_ADJUST_CHANNEL_R         94
//#define SND_SET_CHANNEL_ANGLE_DEFAULT_R  101 // <slot> <angle> 1 1 1 { <angle> 1 1 }

/* 96-127=CURRENT version of 64-95 */


/* automatic sound movement */
#define SND_SET_DX                  128 // <slot> <dx> 1 1 2  { <dx> 1 2 }
#define SND_SET_DZ                  129 // <slot> <dz> 1 1 2  { <dz> 1 2 }
#define SND_SET_DY                  130 // <slot> <dy> 1 1 2  { <dz> 1 2 }

#define SND_ECHO_ON                 131 // <slot> <dt> <fvolume> 1 1 2 1
#define SND_ECHO_OFF                132 // <slot> 1 1

#define SND_DOPPLER_ON              133 // <slot> 1 1
#define SND_DOPPLER_OFF             134 // <slot> 1 1

#define SND_SET_USE_ENVELOPE        135 // <slot> <env> 1 1 1
#define SND_SET_VOLUME              136 // <slot> <vol> 1 1 1  volume of last started 'slot' on the stack

#define SND_SET_SAMPLE_START        142 // <ch> <index> 1 1 4 The offset in samples at which we start playing (only a looped sound will then play the rest)
#define SND_SET_TIME_START          143 // <ch> <microsec> 1 1 4 The offset in seconds at which we start playing (only a looped sound will then play the rest)



#define SND_SET_CHANNEL_DX          144 // <ch> <dx> 1 1 2    { <dx> 1 2 }
#define SND_SET_CHANNEL_DZ          145 // <ch> <dz> 1 1 2    { <dz> 1 2 }
#define SND_SET_CHANNEL_DY          146 // <ch> <dy> 1 1 2    { <dz> 1 2 }
//with SND_SET_(CHANNEL_)ANGLE also give new radius, and use old radius if r=0?

#define SND_CHANNEL_ECHO_ON         147 // <ch> <dt> <fvolume> 1 1 2 1
#define SND_CHANNEL_ECHO_OFF        148 // <ch> 1 1

#define SND_CHANNEL_DOPPLER_ON      149 // <ch> 1 2
#define SND_CHANNEL_DOPPLER_OFF     150 // <ch> 1 1

#define SND_SET_CHANNEL_ENVELOPE    151 // <ch> <env> 1 1 1
#define SND_SET_CHANNEL_VOLUME      152 // <ch> <vol> 1 1 1  volume of channel ch

#define SND_SET_DEFAULT_SLOT_VOLUME 194 // <ch> <vol> 1 1 1
//#define SND_DEFAULT_VOLUME         ?? /* overall volume */

//#define SND_DEFAULT_CHANNEL_VOLUME ?? /* silly ? */
//#define SND_DEFAULT_AUTO_VOLUME    ?? /* silly ? */
//#define SND_CHANGE_AUTO_VOLUME     ?? /* for the stack sounds, only needed if we don't use channel 0 for the stack */
#define SND_SET_CHANNEL_SAMPLE_START 158 // <ch> <index> 1 1 4 The offset in samples at which we start playing (only a looped sound will then play the rest)
#define SND_SET_CHANNEL_TIME_START  159 // <ch> <microsec> 1 1 4 The offset in seconds at which we start playing (only a looped sound will then play the rest)


// Transforms:
#define SND_ENVELOPE                 ?? /* associate an envelope with the sound */
#define SND_MAKE_ENVELOPE            ?? /* 16 envelopes ? */




/*---------------------------------------------------------------------------*
 *
 *	Other things, callbacks, bitmasks, errors
 *
 *---------------------------------------------------------------------------*/

/*__________________SOUND MODIFICATION BITS___________________*/
// Bits denoting what effects may/must influence a sound:
#define SND_BIT_SET_X                 1
#define SND_BIT_SET_Z                 2
#define SND_BIT_SET_Y                 4
#define SND_BIT_ADJUST_X              8
#define SND_BIT_ADJUST_Z             16
#define SND_BIT_ADJUST_Y             32

#define SND_BIT_SET_OBSERVER_X       64
#define SND_BIT_SET_OBSERVER_Z      128
#define SND_BIT_SET_OBSERVER_Y      256

#define SND_BIT_SET_OBSERVER_ANGLE  512
//#define SND_BIT_OBSERVER_DANGLE      ?? /* remove */

#define SND_BIT_DOPPLER            4096
#define SND_BIT_ECHO               8192
#define SND_BIT_ALL_DOPPLER       16384 /* doppler for all sounds */
#define SND_BIT_ALL_ECHO          32768 /* echo for all sounds */
#define SND_BIT_NOISE           (1<<20) /* put some noise on the sound ?? */
#define SND_BIT_ENVELOPE        (1<<21) /* use envelope */

//Of, om aan te geven wat een transformatie bevat.


/*__________________CALL BACK SELECTION___________________*/
// Selecting what to call back? Or simply ignore in the interface?
// Overhead isn't that big.
#define GSI_EVENT_CHANNEL_START       1
#define GSI_EVENT_CHANNEL_END         2
#define GSI_EVENT_SONG_START          4
#define GSI_EVENT_SONG_END            8
#define GSI_EVENT_SONG_ERROR         16
#define GSI_EVENT_MIDI_INFO          32 //change of pan, volumes etc
#define GSI_EVENT_DEVICE_STATUS      64 //open device /dev/dsp etc ok?


/*__________________CALL BACK___________________*/
/* Errors: */
#define GSI_CALLBACK_OK               0
#define GSI_CALLBACK_ERROR            1 /* general error */
#define GSI_CALLBACK_NO_SUCH_FILE     2 //give the string ?
#define GSI_CALLBACK_BAD_LIST         3 /* eg sounds that can't be loaded ?? */
#define GSI_CALLBACK_BAD_COMMAND      4 /* + command no */
#define GSI_CALLBACK_DSP_BUSY         5
#define GSI_CALLBACK_SYNTH_BUSY       6
#define GSI_CALLBACK_CD_BUSY          7
#define GSI_CALLBACK_NO_PATCH_SPACE  20 /* <patchno> 1 1 */
//Use gsi_set_error_function ?

/* Channel playing etc */
// Callback is only useful for channel playing, not the stack, right?
//#define GSI_CALLBACK_END             32 /* <slot> 1 1 number of the sound that finished */
#define GSI_CALLBACK_CHANNEL_STARTED 32 /* <ch> 1 1 */
#define GSI_CALLBACK_CHANNEL_END     33 /* <ch> 1 1 */
#define GSI_CALLBACK_CHANNEL_ENDALL  34 /* 1 (from a KILLALL command) */

#define GSI_CALLBACK_SONG_STARTED    39
#define GSI_CALLBACK_SONG_END        40

#define GSI_STATUS_MIXER             64 /* <mix ch> <val> 1 1 1 return mixer settings that have changed (possibly by another program) */

#define GSI_CALLBACK_SERVER_KILLED  255 /* callback issued from the interface */


//Should the following be renamed: GSI_FORMAT_8BIT_MULAW... GSI_FORMAT_MONO... ?
//Denoting the dsp mode and sound types
#define GSI_8BIT_MULAW                1
#define GSI_8BIT                      2
#define GSI_16BIT                     3
#define GSI_4BIT_ADPCM                4 /* ?? CCITT G.721 ADPCM, only for files, output cannot usually be this format */
#define GSI_3BIT_ADPCM                5 /* CCITT G.723 ADPCM */
#define GSI_5BIT_ADPCM                6 /* CCITT G.723 ADPCM */


//Dsp channels
#define GSI_MONO                      1
#define GSI_STEREO                    2

//Mixer channels
#define GSI_MASTER_VOLUME             0
#define GSI_PCM_VOLUME                1
#define GSI_SYNTH_VOLUME              2
#define GSI_CD_VOLUME                 3
// External midi ?


/*__________________PROGRAM SIDE C/C++ STRUCTURE___________________*/
// This should go into gsi.h when finished.

// This is just a design which is not used at the moment. A function
//      gsi_play (struct gsi_sound_item *s, int effect_bits)
// can easily be added to allow starting or changing a sound using
// such structs (the effect bits, see gsi_commands.h). Also
//      gsi_update (struct gsi_sound_item *s, int effect_bits)
// to update selected parts would be needed.

//#include <gsi/mytypes.h>
#define byte unsigned char
#define uint16 unsigned short
typedef struct gsi_sound_item {
	int  channel;   // -1 for the stack, 0-255 is a channel
	byte type;      //sample, wave (4x), midi
	byte sample_no;
	uint16 freq;

	byte volume_l;  //primary volume adjustments
	byte volume_r;
	int  pan;       // -256 = full left, 256=full right

	int  stereo_x;  //16 bit+8 bit fraction
	int  dist_z;    //16 bit+8 bit fraction
	int  height_y;  //16 bit+8 bit fraction

	// Distances travelled in 1/32 s:
	int  stereo_dx; //8 bit+8 bit fraction
	int  dist_dz;   //8 bit+8 bit fraction
	int  height_dy; //8 bit+8 bit fraction

	int  doppler;   // y/n
	int  echo;      // y/n -> effect_bits?
	int  echo_dt;

	int  stereo_flip; // y/n

	int  repeat;    // y/n
	int  envelope;  // number, -1=none. volume/freq apart?
	int  effect_flags; //which (observer?) transforms affect the sound?

//	char *sample_file_name; //niet nodig -> sample_no
} gsi_sound_item;
#undef byte
#undef uint16



/*---------------------------------------------------------------------------*
 * DESCRIPTION
 * This is only useful for devel work at the moment (very little explanation):
 * Transformations: should these be set per play command or applied each time
 * when a sample/wave is played?
 * 
 * Sound transforms:
 * (current 3d defaults: x=0, z=64, y=0 )
 * 
 * META:
 * allow_change: a bit array with permissions that any of the following may be
 * applied:
 * 
 * DIRECT:
 *    set_volume                       B0
 *    set_noise                        B1
 *    set_echo                         B2
 *    set_doppler (on/off)             B3
 * 
 * INDIRECT 3D:
 *    distance (z)                     B4
 *    distance speed (dz)              B5
 *    stereo pos (x)                   B6
 *    stereo speed (dx)                B7
 *    height (y)                       B8
 *    dheight (dy)                     B9
 *    observer distance (o_z)          B10
 *    observer distance speed (o_dz)   B11
 *    observer stereo pos (o_x)        B12
 *    observer stereo pos speed (o_dx) B13
 *    observer height (o_y)            B14
 *    observer stereo pos speed (o_dy) B15
 *
 * 
 *    observer_flip (inv_x)            B16
 *    (gecombineerd met volume setting geeft dit 360 graden rotatie v.d.
 *     waarnemer)
 * 
 *    observer_angle                   B17
 * 
 *    Euler angles for full 3d motion...
 * 
 * + envelope + sound source motion (z+ stereo) + flip (3d, behind observer)
 * 
 * Sounds moving with observer (sounds in a car eg) must not be affected by
 * observer motion. Also information sounds not related to game action
 * (warning beeps).
 * 
 * internal server stuff:
 *    room_echo, room_echo_dt,
 *
 *---------------------------------------------------------------------------*/

#endif /* _gsi_commands_h */
